package main

import (
	"context"
	"flag"
	"log"

	"git.loafle.net/commons_go/config"
	"git.loafle.net/commons_go/logging"
	"git.loafle.net/overflow/overflow_gateway_web/grpc"
	"git.loafle.net/overflow/overflow_gateway_web/handler"
	"git.loafle.net/overflow/overflow_gateway_web/redis"
	"git.loafle.net/overflow/overflow_gateway_web/server"
	"git.loafle.net/overflow/overflow_gateway_web/subscribe"
)

func main() {
	configPath := flag.String("config", ".", "The path of config file")

	flag.Parse()

	loadConfig(*configPath)
	ctx := context.Background()

	defer logging.Logger.Sync()

	s := server.NewServer(ctx)
	rp := redis.NewPool(ctx)
	grpc.InitializePool(ctx)

	wsh := handler.NewWebSocketHandler(ctx)

	subscribe.Subscribe(ctx, rp.Get())

	s.HandleSocket(config.GetString("handlers.web.entry"), wsh)

	s.ListenAndServe(config.GetString("server.addr"))
}

func loadConfig(path string) {
	config.SetConfigName("config")
	config.AddConfigPath(path)
	err := config.ReadInConfig()
	if nil != err {
		log.Fatalf("config error: %v", err)
	}
}
